from vdiskman import Wholedisk, serialized_call
import xmlrpclib

@serialized_call
def get_file_from_image(image_file_path, file_path):
    """
    Fetch a file from a vdisk.

    @param image_file_path: vdisk.
    @param file_path: file try to fetch.

    @return - A file Binary object when succeed.
    @raise 
    """
    wdisk = Wholedisk(None, image_file_path)
    try:
        mount_path = wdisk.getMountpathByFilename(file_path)
        fd = open(mount_path, 'rb')
        try:
            file = xmlrpclib.Binary(fd.read())
        except Exception, e:
            fd.close()
            wdisk.close()
            raise e
        fd.close()
        wdisk.close()
        return file
    except Exception, e:
        wdisk.close()
        raise e

@serialized_call
def put_file_to_image(image_file_path, file_path, binary_object):
    """
    Put a file to a vdisk. If the file is existed, it will be overwritten.
    If the file is not existed, it will be created.

    @param image_file_path: vdisk path
    @param file_path: file try to put.
    @param binary_object: Binary object. Base64 coded.

    @return - "" when succeed.
    @raise
    """
    wdisk = Wholedisk(None, image_file_path)
    try:
        mount_path = wdisk.getMountpathByFilename(file_path)
        fd = open(mount_path, 'w')
        try:
            fd.write(binary_object.data)
        except Exception, e:
            fd.close()
            wdisk.close()
            raise e
        fd.close()
        wdisk.close()
        return ""
    except Exception, e:
        wdisk.close()
        raise e

