import os
import errno
import logging
from vdiskman import Wholedisk

from OVSDB import db_get_real_vm_path
from OVSXXenVMConfig import xen_unmarshall_cfg
from OVSFileLock import FileLock
from OVSConstants import CONFIG_DIR

file1 = '/etc/sysconfig/oraclevm-template'
file2 = '/oracelvm-template.sh'
wdisk1 = None
wdisk2 = None
vm_lock = None

def D(s):
    logging.debug("[templatetype] %s" % s)
    
def _mkdir_p(path):
    try:
        os.makedirs(path)
    except OSError, e: # Python >2.5
        if e.errno == errno.EEXIST:
            pass
        else:
            raise e

def guess_template_type(vm_path):
    real_vm_path = db_get_real_vm_path(vm_path)
    vm_cfg_file = os.path.join(real_vm_path, 'vm.cfg')
    vm_lock_file = os.path.join(CONFIG_DIR, real_vm_path.lstrip('/'))
    _mkdir_p(vm_lock_file)
    vm_lock_file = os.path.join(vm_lock_file, 'vm.lock')
    cfg = xen_unmarshall_cfg(vm_cfg_file)
    all_disks = cfg["disk"]
    vdisks = [ disk.split(':')[1].split(',')[0] for disk in all_disks if disk.startswith("file:") ]
    global wdisk1, wdisk2, vm_lock
    def close():
        global wdisk1, wdisk2, vm_lock
        if wdisk1:
            D("close wdisk1 %s" % wdisk1)
            wdisk1.close()
            wdisk1 = None
        if wdisk2:
            D("close wdisk2 %s" % wdisk2)
            wdisk2.close()
            wdisk2 = None
        if vm_lock:
            D("close vm lock.")
            vm_lock.release()
    def appendconf(filename):
        f = open(filename, 'a')
        try:
            D("append to %s" % filename)
            f.write('\nTEMPLATE_CONFIG_SCRIPT=/bin/true\n')
        finally:
            D("close %s" % filename)
            f.close()

    if len(vdisks) > 0:
        vm_lock = FileLock(vm_lock_file)
        D("Acquire vm lock %s" % vm_lock_file)
        if not vm_lock.acquire():
            err = "Acquire vm lock %s failed." % vm_lock_file
            raise Exception(err)
        
        """
        2. check /etc/sysconfig/oraclevm-template
        - if the file exists, continue
        - else exit. ### this is not a JeOS template
        """
        wdisk1 = Wholedisk(None, vdisks[0])
        try:
            syscfgvm_path = wdisk1.getMountpathByFilename(file1)
            if os.path.exists(syscfgvm_path):
                """
                3. check TEMPLATE_CONFIG_SCRIPT line in oraclevm-template
                - if the line is commented out, continue
                - else, exit  ### this is a template using custom configuration script
                """
                D("open %s" % syscfgvm_path)
                f = open(syscfgvm_path, 'rb')
                found = False
                try:
                    for line in f.readlines():
                        if line.find('TEMPLATE_CONFIG_SCRIPT') != -1:
                            if not line.lstrip().startswith('#'):
                                found = True
                                break
                finally:
                    f.close()

                if found:
                    close()
                    return "custom"

                if len(vdisks) > 1:
                    """
                    4. check if the 2nd disk defined in vm.cfg
                    - no, continue
                    - yes, mount the 2nd disk as /u01 in the 1st disk
                    5. check if /u01/oracelvm-template.sh in the 1st disk
                    - if the file exists, exit  ### this a product template using default configuration script.
                    - else, this an OEL jeos template, add line "TEMPLATE_CONFIG_SCRIPT=/bin/true" in /etc/sysconfig/oraclevm-template
                    """
                    wdisk2 = Wholedisk(None, vdisks[1])
                    vmtempsh2 = wdisk2.getMountpathByFilename('/oracelvm-template.sh')
                    if os.path.exists(vmtempsh2):
                        close()
                        return "default"
                    else:
                        appendconf(syscfgvm_path)
                        close()
                        return "OEL JeOS"
                else:
                    vmtempsh1 = wdisk1.getMountpathByFilename('/u01/oracelvm-template.sh')
                    if os.path.exists(vmtempsh1):
                        return "default"
                    else:
                        appendconf(syscfgvm_path)
                        close()
                        return "OEL JeOS"
            else:
                close()
                return "Not JeOS"
        except Exception, e:
            close()
            raise e
