# Copyright (C) 2009 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.

import unittest

import sys
import os
import xmlrpclib
import time

sys.path.append(os.path.join(os.path.abspath(os.path.dirname(__file__)), '..'))

from utils.do_rpc import get_local_agt_url
from OVSWrappers import *

class Test(unittest.TestCase):
    agt_url = get_local_agt_url()
    sp = xmlrpclib.ServerProxy(agt_url)
    image_file = '/OVS/running_pool/50_oel5u3_1/System.img'
    image_file_nonexisting = '/OVS/running_pool/a_image_file_should_not_here'
    filename = '/etc/sysconfig/vdiskman_test'
    filename_nonexisting = '/tmp/a_file_should_not_here'
    data = ''

    def setUp(self):
        self.data = "%s: vdiskman test data" % time.asctime()
        file = xmlrpclib.Binary(self.data)
        self.sp.put_file_to_image(self.image_file, self.filename, file)

    def tearDown(self):
        self.sp.utl_set_server_mode('normal')

    def testget_file_from_image(self):
        file = self.sp.get_file_from_image(self.image_file, self.filename)
        self.assertEqual(self.data, file.data)
        print file.data

    def testget_file_from_image_nonexisting(self):
        try:
            file = self.sp.get_file_from_image(self.image_file, self.filename_nonexisting)
        except xmlrpclib.Fault, fault:
            self.assertTrue(fault.faultCode == 1)
            self.assertTrue(fault.faultString.startswith('exceptions.IOError:[Errno 2] No such file or directory:'))
        else:
            raise

    def testget_file_from_image_img_nonexisting(self):
        try:
            file = self.sp.get_file_from_image(self.image_file_nonexisting,
                self.filename)
        except xmlrpclib.Fault, fault:
            self.assertTrue(fault.faultCode == 1)
            self.assertEqual(fault.faultString, 'exceptions.RuntimeError:Failure setting up loop dev')
        else:
            raise

    def testput_file_to_image_img_nonexisting(self):
        try:
            file = xmlrpclib.Binary(self.data)
            self.sp.put_file_to_image(self.image_file_nonexisting, self.filename, file)
        except xmlrpclib.Fault, fault:
            self.assertTrue(fault.faultCode == 1)
            self.assertEqual(fault.faultString, 'exceptions.RuntimeError:Failure setting up loop dev')
        else:
            raise

if __name__ == "__main__":
    sys.argv = ['', '-v']
    unittest.main()
